@@ -96,6 +96,7 @@
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
+#include <ctype.h>
 
 #include "gpsd_config.h"
 #include "gps.h"
@@ -410,33 +411,25 @@ static void update_compass_panel(struct gps_data_t *gpsdata)
 static void update_gps_panel(struct gps_data_t *gpsdata)
 /* This gets called once for each new GPS sentence. */
 {
-    int i, j, n;
     int newstate;
     char scr[128], *s;
-    bool usedflags[MAXCHANNELS];
-
-    /* must build bit vector of which statellites are used from list */
-    for (i = 0; i < MAXCHANNELS; i++) {
-	usedflags[i] = false;
-	for (j = 0; j < gpsdata->satellites_used; j++)
-	    if (gpsdata->used[j] == gpsdata->PRN[i])
-		usedflags[i] = true;
-    }
 
     /* This is for the satellite status display.  Originally lifted from
      * xgps.c.  Note that the satellite list may be truncated based on
      * available screen size, or may only show satellites used for the
      * fix.  */
     if (gpsdata->satellites_visible != 0) {
+	int i;
 	if (display_sats >= MAX_POSSIBLE_SATS) {
 	    for (i = 0; i < MAX_POSSIBLE_SATS; i++) {
 		if (i < gpsdata->satellites_visible) {
 		    (void)snprintf(scr, sizeof(scr),
 				   " %3d    %02d    %03d    %02d      %c",
-				   gpsdata->PRN[i],
-				   gpsdata->elevation[i], gpsdata->azimuth[i],
-				   (int)gpsdata->ss[i],
-				   usedflags[i] ? 'Y' : 'N');
+				   gpsdata->skyview[i].PRN,
+				   gpsdata->skyview[i].elevation,
+				   gpsdata->skyview[i].azimuth,
+				   (int)gpsdata->skyview[i].ss,
+				   gpsdata->skyview[i].used ? 'Y' : 'N');
 		} else {
 		    (void)strlcpy(scr, "", sizeof(scr));
 		}
@@ -444,18 +437,19 @@ static void update_gps_panel(struct gps_data_t *gpsdata)
 				SATELLITES_WIDTH - 3, scr);
 	    }
 	} else {
-	    n = 0;
+	    int n = 0;
 	    for (i = 0; i < MAX_POSSIBLE_SATS; i++) {
 		if (n < display_sats) {
 		    if ((i < gpsdata->satellites_visible)
-			&& ((gpsdata->used[i] != 0)
+			&& (gpsdata->skyview[i].used
 			    || (gpsdata->satellites_visible <= display_sats))) {
 			(void)snprintf(scr, sizeof(scr),
 				       " %3d    %02d    %03d    %02d      %c",
-				       gpsdata->PRN[i], gpsdata->elevation[i],
-				       gpsdata->azimuth[i],
-				       (int)gpsdata->ss[i],
-				       gpsdata->used[i] ? 'Y' : 'N');
+				       gpsdata->skyview[i].PRN,
+				       gpsdata->skyview[i].elevation,
+				       gpsdata->skyview[i].azimuth,
+				       (int)gpsdata->skyview[i].ss,
+				       gpsdata->skyview[i].used ? 'Y' : 'N');
 			(void)mvwprintw(satellites, n + 2, 1, "%-*s",
 					SATELLITES_WIDTH - 3, scr);
 			n++;
@@ -546,17 +540,16 @@ static void update_gps_panel(struct gps_data_t *gpsdata)
 	(void)snprintf(scr, sizeof(scr), "OFFLINE");
     } else {
 	newstate = gpsdata->fix.mode;
+	/*
+	 * DGPS
+	 */
 	switch (gpsdata->fix.mode) {
 	case MODE_2D:
-	    (void)snprintf(scr, sizeof(scr), "2D %sFIX (%d secs)",
-			   (gpsdata->status ==
-			    STATUS_DGPS_FIX) ? "DIFF " : "",
+	    (void)snprintf(scr, sizeof(scr), "2D FIX (%d secs)",
 			   (int)(time(NULL) - status_timer));
 	    break;
 	case MODE_3D:
-	    (void)snprintf(scr, sizeof(scr), "3D %sFIX (%d secs)",
-			   (gpsdata->status ==
-			    STATUS_DGPS_FIX) ? "DIFF " : "",
+	    (void)snprintf(scr, sizeof(scr), "3D FIX (%d secs)",
 			   (int)(time(NULL) - status_timer));
 	    break;
 	default:
@@ -639,9 +632,14 @@ static void update_gps_panel(struct gps_data_t *gpsdata)
     }
 
     /* Be quiet if the user requests silence. */
+    /*@-modobserver@*/
     if (!silent_flag && raw_flag && (s = (char *)gps_data(gpsdata)) != NULL) {
-	(void)waddstr(messages, s);
+	char *p;
+	for (p = s + strlen(s); --p > s && isspace((unsigned char) *p); *p = '\0')
+	    ;
+	(void)wprintw(messages, "%s\n", s);
     }
+    /*@+modobserver@*/
 
     /* Reset the status_timer if the state has changed. */
     if (newstate != state) {
@@ -682,7 +680,6 @@ static void usage(char *prog)
 int main(int argc, char *argv[])
 {
     int option;
-    int c;
     unsigned int flags = WATCH_ENABLE;
 
     /*@ -observertrans @*/
@@ -808,6 +805,8 @@ int main(int argc, char *argv[])
 
     /* heart of the client */
     for (;;) {
+	int c;
+
 	if (!gps_waiting(&gpsdata, 5000000)) {
 	    die(GPS_TIMEOUT);
 	} else {
