@@ -20,6 +20,7 @@ PERMISSIONS
 #include <stddef.h>
 
 #include "gpsd.h"
+#include "strfuncs.h"
 #ifdef SOCKET_EXPORT_ENABLE
 #include "gps_json.h"
 
@@ -40,8 +41,6 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
 	{"class",  t_check,   .dflt.check = "TPV"},
 	{"device", t_string,  .addr.string = gpsdata->dev.path,
 			         .len = sizeof(gpsdata->dev.path)},
-	{"tag",    t_string,  .addr.string = gpsdata->tag,
-			         .len = sizeof(gpsdata->tag)},
 	{"time",   t_time,    .addr.real = &gpsdata->fix.time,
 			         .dflt.real = NAN},
 	{"time",   t_real,    .addr.real = &gpsdata->fix.time,
@@ -91,8 +90,6 @@ static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
 	{"class",  t_check,   .dflt.check = "GST"},
 	{"device", t_string,  .addr.string = gpsdata->dev.path,
 			         .len = sizeof(gpsdata->dev.path)},
-	{"tag",    t_string,  .addr.string = gpsdata->tag,
-			         .len = sizeof(gpsdata->tag)},
 	{"time",   t_time,    .addr.real = &gpsdata->gst.utctime,
 			         .dflt.real = NAN},
 	{"time",   t_real,    .addr.real = &gpsdata->gst.utctime,
@@ -122,15 +119,15 @@ static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
 static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 			 /*@null@*/ const char **endptr)
 {
-    bool usedflags[MAXCHANNELS];
     /*@ -fullinitblock @*/
-    const struct json_attr_t json_attrs_2_1[] = {
+    /*@-type@*//* STRUCTARRAY confuses splint */
+    const struct json_attr_t json_attrs_satellites[] = {
 	/* *INDENT-OFF* */
-	{"PRN",	   t_integer, .addr.integer = gpsdata->PRN},
-	{"el",	   t_integer, .addr.integer = gpsdata->elevation},
-	{"az",	   t_integer, .addr.integer = gpsdata->azimuth},
-	{"ss",	   t_real,    .addr.real = gpsdata->ss},
-	{"used",   t_boolean, .addr.boolean = usedflags},
+	{"PRN",	   t_integer, STRUCTOBJECT(struct satellite_t, PRN)},
+	{"el",	   t_integer, STRUCTOBJECT(struct satellite_t, elevation)},
+	{"az",	   t_integer, STRUCTOBJECT(struct satellite_t, azimuth)},
+	{"ss",	   t_real,    STRUCTOBJECT(struct satellite_t, ss)},
+	{"used",   t_boolean, STRUCTOBJECT(struct satellite_t, used)},
 	/* *INDENT-ON* */
 	{NULL},
     };
@@ -139,8 +136,6 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 	{"class",      t_check,   .dflt.check = "SKY"},
 	{"device",     t_string,  .addr.string  = gpsdata->dev.path,
 	                             .len = sizeof(gpsdata->dev.path)},
-	{"tag",	       t_string,  .addr.string  = gpsdata->tag,
-	                             .len = sizeof(gpsdata->tag)},
 	{"time",       t_time,    .addr.real = &gpsdata->skyview_time,
 	      	                     .dflt.real = NAN},
 	{"time",       t_real,    .addr.real = &gpsdata->skyview_time,
@@ -159,10 +154,10 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 	                             .dflt.real = NAN},
 	{"gdop",       t_real,    .addr.real    = &gpsdata->dop.gdop,
 	                             .dflt.real = NAN},
-	{"satellites", t_array,   .addr.array.element_type = t_object,
-				     .addr.array.arr.objects.subtype=json_attrs_2_1,
-	                             .addr.array.maxlen = MAXCHANNELS,
-	                             .addr.array.count = &gpsdata->satellites_visible},
+	{"satellites", t_array,
+	                           STRUCTARRAY(gpsdata->skyview,
+					 json_attrs_satellites,
+					 &gpsdata->satellites_visible)},
 	{NULL},
 	/* *INDENT-ON* */
     };
@@ -170,8 +165,8 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
     int status, i, j;
 
     for (i = 0; i < MAXCHANNELS; i++) {
-	gpsdata->PRN[i] = 0;
-	usedflags[i] = false;
+	gpsdata->skyview[i].PRN = 0;
+	gpsdata->skyview[i].used = false;
     }
 
     status = json_read_object(buf, json_attrs_2, endptr);
@@ -180,12 +175,10 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 
     gpsdata->satellites_used = 0;
     gpsdata->satellites_visible = 0;
-    (void)memset(gpsdata->used, '\0', sizeof(gpsdata->used));
     for (i = j = 0; i < MAXCHANNELS; i++) {
-	if(gpsdata->PRN[i] > 0)
+	if(gpsdata->skyview[i].PRN > 0)
 	    gpsdata->satellites_visible++;
-	if (usedflags[i]) {
-	    gpsdata->used[j++] = gpsdata->PRN[i];
+	if (gpsdata->skyview[i].used) {
 	    gpsdata->satellites_used++;
 	}
     }
@@ -202,8 +195,6 @@ static int json_att_read(const char *buf, struct gps_data_t *gpsdata,
 	{"class",    t_check,     .dflt.check = "ATT"},
 	{"device",   t_string,    .addr.string = gpsdata->dev.path,
 			             .len = sizeof(gpsdata->dev.path)},
-	{"tag",      t_string,    .addr.string = gpsdata->tag,
-			             .len = sizeof(gpsdata->tag)},
 	{"heading",  t_real,      .addr.real = &gpsdata->attitude.heading,
 			             .dflt.real = NAN},
 	{"mag_st",   t_character, .addr.character = &gpsdata->attitude.mag_st},
@@ -217,6 +208,8 @@ static int json_att_read(const char *buf, struct gps_data_t *gpsdata,
 			             .dflt.real = NAN},
 	{"yaw_st",   t_character, .addr.character = &gpsdata->attitude.yaw_st},
 
+	{"dip",      t_real,      .addr.real = &gpsdata->attitude.dip,
+			             .dflt.real = NAN},
 	{"mag_len",  t_real,      .addr.real = &gpsdata->attitude.mag_len,
 			             .dflt.real = NAN},
 	{"mag_x",    t_real,      .addr.real = &gpsdata->attitude.mag_x,
@@ -385,10 +378,10 @@ int json_pps_read(const char *buf, struct gps_data_t *gpsdata,
      */
     /*@-usedef@*/
     /*@-type@*//* splint is confused about struct timespec */
-    gpsdata->timedrift.real.tv_sec = (long)real_sec;
-    gpsdata->timedrift.real.tv_nsec = (time_t)real_nsec;
-    gpsdata->timedrift.clock.tv_sec = (long)clock_sec;
-    gpsdata->timedrift.clock.tv_nsec = (time_t)clock_nsec;
+    gpsdata->timedrift.real.tv_sec = (time_t)real_sec;
+    gpsdata->timedrift.real.tv_nsec = (long)real_nsec;
+    gpsdata->timedrift.clock.tv_sec = (time_t)clock_sec;
+    gpsdata->timedrift.clock.tv_nsec = (long)clock_nsec;
     /*@+type@*/
     /*@+usedef@*/
     if (status != 0)
@@ -406,8 +399,7 @@ int libgps_json_unpack(const char *buf,
 
     if (classtag == NULL)
 	return -1;
-#define STARTSWITH(str, prefix)	strncmp(str, prefix, sizeof(prefix)-1)==0
-    if (STARTSWITH(classtag, "\"class\":\"TPV\"")) {
+    if (str_starts_with(classtag, "\"class\":\"TPV\"")) {
 	status = json_tpv_read(buf, gpsdata, end);
 	gpsdata->status = STATUS_FIX;
 	gpsdata->set = STATUS_SET;
@@ -440,45 +432,45 @@ int libgps_json_unpack(const char *buf,
 	if (gpsdata->fix.mode != MODE_NOT_SEEN)
 	    gpsdata->set |= MODE_SET;
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"GST\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"GST\"")) {
 	status = json_noise_read(buf, gpsdata, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= GST_SET;
 	}
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"SKY\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"SKY\"")) {
 	status = json_sky_read(buf, gpsdata, end);
 	if (status == 0)
 	    gpsdata->set |= SATELLITE_SET;
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"ATT\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"ATT\"")) {
 	status = json_att_read(buf, gpsdata, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= ATTITUDE_SET;
 	}
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"DEVICES\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"DEVICES\"")) {
 	status = json_devicelist_read(buf, gpsdata, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= DEVICELIST_SET;
 	}
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"DEVICE\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"DEVICE\"")) {
 	status = json_device_read(buf, &gpsdata->dev, end);
 	if (status == 0)
 	    gpsdata->set |= DEVICE_SET;
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"WATCH\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"WATCH\"")) {
 	status = json_watch_read(buf, &gpsdata->policy, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= POLICY_SET;
 	}
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"VERSION\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"VERSION\"")) {
 	status = json_version_read(buf, gpsdata, end);
 	if (status ==  0) {
 	    gpsdata->set &= ~UNION_SET;
@@ -486,7 +478,7 @@ int libgps_json_unpack(const char *buf,
 	}
 	return status;
 #ifdef RTCM104V2_ENABLE
-    } else if (STARTSWITH(classtag, "\"class\":\"RTCM2\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"RTCM2\"")) {
 	status = json_rtcm2_read(buf,
 				 gpsdata->dev.path, sizeof(gpsdata->dev.path),
 				 &gpsdata->rtcm2, end);
@@ -497,7 +489,7 @@ int libgps_json_unpack(const char *buf,
 	return status;
 #endif /* RTCM104V2_ENABLE */
 #ifdef RTCM104V3_ENABLE
-    } else if (STARTSWITH(classtag, "\"class\":\"RTCM3\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"RTCM3\"")) {
 	status = json_rtcm3_read(buf,
 				 gpsdata->dev.path, sizeof(gpsdata->dev.path),
 				 &gpsdata->rtcm3, end);
@@ -508,7 +500,7 @@ int libgps_json_unpack(const char *buf,
 	return status;
 #endif /* RTCM104V3_ENABLE */
 #ifdef AIVDM_ENABLE
-    } else if (STARTSWITH(classtag, "\"class\":\"AIS\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"AIS\"")) {
 	status = json_ais_read(buf,
 			       gpsdata->dev.path, sizeof(gpsdata->dev.path),
 			       &gpsdata->ais, end);
@@ -518,23 +510,22 @@ int libgps_json_unpack(const char *buf,
 	}
 	return status;
 #endif /* AIVDM_ENABLE */
-    } else if (STARTSWITH(classtag, "\"class\":\"ERROR\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"ERROR\"")) {
 	status = json_error_read(buf, gpsdata, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= ERROR_SET;
 	}
 	return status;
-    } else if (STARTSWITH(classtag, "\"class\":\"PPS\"")) {
+    } else if (str_starts_with(classtag, "\"class\":\"PPS\"")) {
 	status = json_pps_read(buf, gpsdata, end);
 	if (status == 0) {
 	    gpsdata->set &= ~UNION_SET;
-	    gpsdata->set |= TIMEDRIFT_SET;
+	    gpsdata->set |= PPSDRIFT_SET;
 	}
 	return status;
     } else
 	return -1;
-#undef STARTSWITH
 }
 
 /*@+compdef@*/
