@@ -21,6 +21,7 @@
 
 #include "gpsd.h"
 #include "bsd_base64.h"
+#include "strfuncs.h"
 
 #define NTRIP_SOURCETABLE	"SOURCETABLE 200 OK\r\n"
 #define NTRIP_ENDSOURCETABLE	"ENDSOURCETABLE"
@@ -36,7 +37,7 @@
 static /*@null@*/ char *ntrip_field_iterate( /*@null@ */ char *start,
 					     /*@null@*/ char *prev,
 					     const char *eol,
-					     const int debug)
+					     const struct gpsd_errout_t *errout)
 {
     char *s, *t, *u;
 
@@ -58,7 +59,7 @@ static /*@null@*/ char *ntrip_field_iterate( /*@null@ */ char *start,
     if ((t = strstr(t, ";")))
 	*t = '\0';
 
-    gpsd_report(debug, LOG_RAW, "Next Ntrip source table field %s\n", s);
+    gpsd_report(errout, LOG_RAW, "Next Ntrip source table field %s\n", s);
 
     return s;
 }
@@ -68,19 +69,19 @@ static /*@null@*/ char *ntrip_field_iterate( /*@null@ */ char *start,
 /*@ -mustfreefresh @*/
 static void ntrip_str_parse(char *str, size_t len,
 			    /*@out@*/ struct ntrip_stream_t *hold,
-			    const int debug)
+			    const struct gpsd_errout_t *errout)
 {
     char *s, *eol = str + len;
 
     memset(hold, 0, sizeof(*hold));
 
     /* <mountpoint> */
-    if ((s = ntrip_field_iterate(str, NULL, eol, debug)))
+    if ((s = ntrip_field_iterate(str, NULL, eol, errout)))
 	(void)strlcpy(hold->mountpoint, s, sizeof(hold->mountpoint));
     /* <identifier> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <format> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	if (strcasecmp("RTCM 2", s) == 0)
 	    hold->format = fmt_rtcm2;
 	else if (strcasecmp("RTCM 2.0", s) == 0)
@@ -100,41 +101,41 @@ static void ntrip_str_parse(char *str, size_t len,
 	    hold->format = fmt_unknown;
     }
     /* <format-details> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <carrier> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug)))
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout)))
 	hold->carrier = atoi(s);
     /* <nav-system> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <network> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <country> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <latitude> */
     hold->latitude = NAN;
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug)))
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout)))
 	hold->latitude = atof(s);
     /* <longitude> */
     hold->longitude = NAN;
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug)))
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout)))
 	hold->longitude = atof(s);
     /* <nmea> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	hold->nmea = atoi(s);
     }
     /* <solution> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <generator> */
-    s = ntrip_field_iterate(NULL, s, eol, debug);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
     /* <compr-encryp> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	if (strcasecmp("none", s) == 0)
 	    hold->compr_encryp = cmp_enc_none;
 	else
 	    hold->compr_encryp = cmp_enc_unknown;
     }
     /* <authentication> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	if (strcasecmp("N", s) == 0)
 	    hold->authentication = auth_none;
 	else if (strcasecmp("B", s) == 0)
@@ -145,15 +146,15 @@ static void ntrip_str_parse(char *str, size_t len,
 	    hold->authentication = auth_unknown;
     }
     /* <fee> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	hold->fee = atoi(s);
     }
     /* <bitrate> */
-    if ((s = ntrip_field_iterate(NULL, s, eol, debug))) {
+    if ((s = ntrip_field_iterate(NULL, s, eol, errout))) {
 	hold->bitrate = atoi(s);
     }
     /* ...<misc> */
-    while ((s = ntrip_field_iterate(NULL, s, eol, debug)));
+    while ((s = ntrip_field_iterate(NULL, s, eol, errout)));
 }
 
 static int ntrip_sourcetable_parse(struct gps_device_t *device)
@@ -164,7 +165,7 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
     bool sourcetable = false;
     bool match = false;
     char buf[BUFSIZ];
-    size_t blen = BUFSIZ;
+    size_t blen = sizeof(buf);
     int fd = device->gpsdata.gps_fd;
 
     for (;;) {
@@ -183,12 +184,12 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 	    if (match) {
 		return 1;
 	    }
-	    gpsd_report(device->context->debug, LOG_ERROR, 
+	    gpsd_report(&device->context->errout, LOG_ERROR, 
 			"ntrip stream read error %d on fd %d\n",
 			errno, fd);
 	    return -1;
 	} else if (rlen == 0) { // server closed the connection
-	    gpsd_report(device->context->debug, LOG_ERROR,
+	    gpsd_report(&device->context->errout, LOG_ERROR,
 			"ntrip stream unexpected close %d on fd %d during sourcetable read\n",
 			errno, fd);
 	    return -1;
@@ -197,21 +198,20 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 	line = buf;
 	rlen = len += rlen;
 
-	gpsd_report(device->context->debug, LOG_RAW,
+	gpsd_report(&device->context->errout, LOG_RAW,
 		    "Ntrip source table buffer %s\n", buf);
 
 	sourcetable = device->ntrip.sourcetable_parse;
 	if (!sourcetable) {
 	    /* parse SOURCETABLE */
-	    if (strncmp(line, NTRIP_SOURCETABLE, strlen(NTRIP_SOURCETABLE)) ==
-		    0) {
+	    if (str_starts_with(line, NTRIP_SOURCETABLE)) {
 		sourcetable = true;
 		device->ntrip.sourcetable_parse = true;
 		llen = (ssize_t) strlen(NTRIP_SOURCETABLE);
 		line += llen;
 		len -= llen;
 	    } else {
-		gpsd_report(device->context->debug, LOG_WARN,
+		gpsd_report(&device->context->errout, LOG_WARN,
 			    "Received unexpexted Ntrip reply %s.\n",
 			    buf);
 		return -1;
@@ -220,16 +220,14 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 
 	while (len > 0) {
 	    /* parse ENDSOURCETABLE */
-	    if (strncmp
-		    (line, NTRIP_ENDSOURCETABLE,
-		     strlen(NTRIP_ENDSOURCETABLE)) == 0)
+	    if (str_starts_with(line, NTRIP_ENDSOURCETABLE))
 		goto done;
 
 	    /* coverity[string_null] - nul-terminated by previous memset */
 	    if (!(eol = strstr(line, NTRIP_BR)))
 		break;
 
-	    gpsd_report(device->context->debug, LOG_DATA,
+	    gpsd_report(&device->context->errout, LOG_DATA,
 			"next Ntrip source table line %s\n", line);
 
 	    *eol = '\0';
@@ -238,21 +236,21 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 	    /* todo: parse headers */
 
 	    /* parse STR */
-	    if (strncmp(line, NTRIP_STR, strlen(NTRIP_STR)) == 0) {
+	    if (str_starts_with(line, NTRIP_STR)) {
 		ntrip_str_parse(line + strlen(NTRIP_STR),
 				(size_t) (llen - strlen(NTRIP_STR)), 
-				&hold, device->context->debug);
+				&hold, &device->context->errout);
 		if (strcmp(device->ntrip.stream.mountpoint, hold.mountpoint) == 0) {
 		    /* todo: support for RTCM 3.0, SBAS (WAAS, EGNOS), ... */
 		    if (hold.format == fmt_unknown) {
-			gpsd_report(device->context->debug, LOG_ERROR,
+			gpsd_report(&device->context->errout, LOG_ERROR,
 				"Ntrip stream %s format not supported\n",
 				line);
 			return -1;
 		    }
 		    /* todo: support encryption and compression algorithms */
 		    if (hold.compr_encryp != cmp_enc_none) {
-			gpsd_report(device->context->debug, LOG_ERROR,
+			gpsd_report(&device->context->errout, LOG_ERROR,
 				"Ntrip stream %s compression/encryption algorithm not supported\n",
 				line);
 			return -1;
@@ -260,7 +258,7 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 		    /* todo: support digest authentication */
 		    if (hold.authentication != auth_none
 			    && hold.authentication != auth_basic) {
-			gpsd_report(device->context->debug, LOG_ERROR,
+			gpsd_report(&device->context->errout, LOG_ERROR,
 				"Ntrip stream %s authentication method not supported\n",
 				line);
 			return -1;
@@ -282,15 +280,15 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 		 * find nearest stream if user hasn't provided one */
 	    }
 	    /* todo: parse CAS */
-	    /* else if (strncmp(line, NTRIP_CAS, strlen(NTRIP_CAS))==0); */
+	    /* else if (str_starts_with(line, NTRIP_CAS)); */
 
 	    /* todo: parse NET */
-	    /* else if (strncmp(line, NTRIP_NET, strlen(NTRIP_NET))==0); */
+	    /* else if (str_starts_with(line, NTRIP_NET)); */
 
 	    llen += strlen(NTRIP_BR);
 	    line += llen;
 	    len -= llen;
-	    gpsd_report(device->context->debug, LOG_RAW,
+	    gpsd_report(&device->context->errout, LOG_RAW,
 		    "Remaining Ntrip source table buffer %zd %s\n", len,
 		    line);
 	}
@@ -307,7 +305,7 @@ done:
 }
 
 static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
-				  const int debug)
+				  struct gpsd_errout_t *errout)
 {
     int dsock;
     ssize_t r;
@@ -315,10 +313,10 @@ static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
 
     dsock = netlib_connectsock(AF_UNSPEC, stream->url, stream->port, "tcp");
     if (dsock < 0) {
-	gpsd_report(debug, LOG_ERROR, "ntrip stream connect error %d in req probe\n", dsock);
+	gpsd_report(errout, LOG_ERROR, "ntrip stream connect error %d in req probe\n", dsock);
 	return -1;
     }
-    gpsd_report(debug, LOG_SPIN, "ntrip stream for req probe connected on fd %d\n", dsock);
+    gpsd_report(errout, LOG_SPIN, "ntrip stream for req probe connected on fd %d\n", dsock);
     (void)snprintf(buf, sizeof(buf),
 	    "GET / HTTP/1.1\r\n"
 	    "User-Agent: NTRIP gpsd/%s\r\n"
@@ -327,7 +325,7 @@ static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
 	    "\r\n", VERSION, stream->url);
     r = write(dsock, buf, strlen(buf));
     if (r != (ssize_t)strlen(buf)) {
-	gpsd_report(debug, LOG_ERROR, 
+	gpsd_report(errout, LOG_ERROR, 
 		    "ntrip stream write error %d on fd %d during probe request %zd\n",
 		    errno, dsock, r);
 	(void)close(dsock);
@@ -364,19 +362,19 @@ static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
 /* *INDENT-ON* */
 
 static int ntrip_stream_get_req(const struct ntrip_stream_t *stream,
-				const int debug)
+				const struct gpsd_errout_t *errout)
 {
     int dsock;
     char buf[BUFSIZ];
 
     dsock = netlib_connectsock(AF_UNSPEC, stream->url, stream->port, "tcp");
     if (BAD_SOCKET(dsock)) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		    "ntrip stream connect error %d\n", dsock);
 	return -1;
     }
 
-    gpsd_report(debug, LOG_SPIN,
+    gpsd_report(errout, LOG_SPIN,
 		"netlib_connectsock() returns socket on fd %d\n",
 		dsock);
 
@@ -389,7 +387,7 @@ static int ntrip_stream_get_req(const struct ntrip_stream_t *stream,
 	    "Connection: close\r\n"
 	    "\r\n", stream->mountpoint, VERSION, stream->url, stream->authStr);
     if (write(dsock, buf, strlen(buf)) != (ssize_t) strlen(buf)) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		    "ntrip stream write error %d on fd %d during get request\n", errno,
 		    dsock);
 	(void)close(dsock);
@@ -399,7 +397,8 @@ static int ntrip_stream_get_req(const struct ntrip_stream_t *stream,
 }
 
 static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream,
-				  const int dsock, const int debug)
+				  const int dsock, 
+				  const struct gpsd_errout_t *errout)
 {
 /*@-nullpass@*/
     char buf[BUFSIZ];
@@ -408,7 +407,7 @@ static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream,
     while (read(dsock, buf, sizeof(buf) - 1) == -1) {
 	if (errno == EINTR)
 	    continue;
-	gpsd_report(debug, LOG_ERROR, 
+	gpsd_report(errout, LOG_ERROR, 
 		    "ntrip stream read error %d on fd %d during get rsp\n", errno,
 		    dsock);
 	goto close;
@@ -417,21 +416,21 @@ static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream,
     /* parse 401 Unauthorized */
     /* coverity[string_null] - guaranteed terminated by the memset above */
     if (strstr(buf, NTRIP_UNAUTH)!=NULL) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		"not authorized for Ntrip stream %s/%s\n", stream->url,
 		stream->mountpoint);
 	goto close;
     }
     /* parse SOURCETABLE */
     if (strstr(buf, NTRIP_SOURCETABLE)!=NULL) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		"Broadcaster doesn't recognize Ntrip stream %s:%s/%s\n",
 		stream->url, stream->port, stream->mountpoint);
 	goto close;
     }
     /* parse ICY 200 OK */
     if (strstr(buf, NTRIP_ICY)==NULL) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		"Unknown reply %s from Ntrip service %s:%s/%s\n", buf,
 		stream->url, stream->port, stream->mountpoint);
 	goto close;
@@ -468,7 +467,7 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 	    device->ntrip.works = false;
 	    device->ntrip.sourcetable_parse = false;
 	    device->ntrip.stream.set = false;
-	    (void)strlcpy(tmp, caster, strlen(caster) + 1);
+	    (void)strlcpy(tmp, caster, sizeof(t));
 
 	    /*@ -boolops @*/
 	    if ((amp = strchr(tmp, '@')) != NULL) {
@@ -478,7 +477,7 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 		    tmp = amp + 1;
 		    url = tmp;
 		} else {
-		    gpsd_report(device->context->debug, LOG_ERROR,
+		    gpsd_report(&device->context->errout, LOG_ERROR,
 			    "can't extract user-ID and password from %s\n",
 			    caster);
 		    device->ntrip.conn_state = ntrip_conn_err;
@@ -491,7 +490,7 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 		stream = slash + 1;
 	    } else {
 		/* todo: add autoconnect like in dgpsip.c */
-		gpsd_report(device->context->debug, LOG_ERROR,
+		gpsd_report(&device->context->errout, LOG_ERROR,
 			    "can't extract Ntrip stream from %s\n",
 			    caster);
 		device->ntrip.conn_state = ntrip_conn_err;
@@ -528,7 +527,7 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 			      sizeof(device->ntrip.stream.port));
 
 	    ret = ntrip_stream_req_probe(&device->ntrip.stream,
-					 device->context->debug);
+					 &device->context->errout);
 	    if (ret == -1) {
 		device->ntrip.conn_state = ntrip_conn_err;
 		return -1;
@@ -546,12 +545,12 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 		return ret;
 	    }
 	    (void)close(device->gpsdata.gps_fd);
-	    if (ntrip_auth_encode(&device->ntrip.stream, device->ntrip.stream.credentials, device->ntrip.stream.authStr, 128) != 0) {
+	    if (ntrip_auth_encode(&device->ntrip.stream, device->ntrip.stream.credentials, device->ntrip.stream.authStr, sizeof(device->ntrip.stream.authStr)) != 0) {
 		device->ntrip.conn_state = ntrip_conn_err;
 		return -1;
 	    }
 	    ret = ntrip_stream_get_req(&device->ntrip.stream,
-				       device->context->debug);
+				       &device->context->errout);
 	    if (ret == -1) {
 		device->ntrip.conn_state = ntrip_conn_err;
 		return -1;
@@ -562,7 +561,7 @@ int ntrip_open(struct gps_device_t *device, char *caster)
 	case ntrip_conn_sent_get:
 	    ret = ntrip_stream_get_parse(&device->ntrip.stream,
 					 device->gpsdata.gps_fd,
-					 device->context->debug);
+					 &device->context->errout);
 	    if (ret == -1) {
 		device->ntrip.conn_state = ntrip_conn_err;
 		return -1;
@@ -598,9 +597,9 @@ void ntrip_report(struct gps_context_t *context,
 	    gpsd_position_fix_dump(gps, buf, sizeof(buf));
 	    if (write(caster->gpsdata.gps_fd, buf, strlen(buf)) ==
 		    (ssize_t) strlen(buf)) {
-		gpsd_report(context->debug, LOG_IO, "=> dgps %s\n", buf);
+		gpsd_report(&context->errout, LOG_IO, "=> dgps %s\n", buf);
 	    } else {
-		gpsd_report(context->debug, LOG_IO, "ntrip report write failed\n");
+		gpsd_report(&context->errout, LOG_IO, "ntrip report write failed\n");
 	    }
 	}
     }
